﻿using System;
using System.Linq;
using System.Net;
using System.Windows.Forms;

namespace PLCi_Simple_Watch_Window
{
    public partial class WatchWindow : Form
    {
        //   Timer thread triggers application to update on screen variable values every 50 ms - see OnTimer method below
        static int updateRate = 50;
        public static System.Timers.Timer UpdateTimer = new System.Timers.Timer() { Interval = updateRate };
        public string ipAddress;                        //   IP Address of current connection
        public bool connected = false;                  //   Connection status
        public MyPLCi MPiecComm = new MyPLCi();

        // start Windows Forms application
        public WatchWindow()
        {
            InitializeComponent();  
        }

        //   Start the timer thread to update the screen once the window has loaded
        private void WatchWindow_Load(object sender, EventArgs e)
        {
            UpdateTimer.Start();
            UpdateTimer.Elapsed += OnTimer;
        }

        //   The OnTimer method is executed when the timer thread reaches its time limit
        public void OnTimer(Object source, System.Timers.ElapsedEventArgs e)  
        {
            //   When the timer goes off, stop it...
            UpdateTimer.Stop();
            //   Only update the online values if connected to PLC
            if (connected)
            {
                myStruct testStruct = MPiecComm.ReadStruct();        //   Get current values from PLC
                //   Update the text boxes displaying the variables in SimpleMachine
                SetTextX(testStruct.XPos.ToString("0.##"));     //   Display formatting 0.## for two decimal places
                SetTextY(testStruct.YPos.ToString("0.##"));
                SetTextZ(testStruct.ZPos.ToString("0.##"));
                SetTextFeed(testStruct.Feed.ToString());
                SetTextStart(testStruct.CycleStart.ToString());
                SetTextStop(testStruct.CycleStop.ToString());
                SetTextStartMove(testStruct.StartMove.ToString());
            }
            //   Restart the timer
            UpdateTimer.Start();
        }

        //   Connect_Click connects / disconnects from the PLC depending on the current state when the Connect/Disconnect button in the UI is pressed
        private void Connect_Click(object sender, EventArgs e)
        {
            if (connected)
            {
                //   If PLC is already connected, then the button acts as a disconnect button
                MPiecComm.Disconnect();
                connected = false;
                ConnectionStatus.Text = "Disconnected";
                ConnectionStatus.ForeColor = System.Drawing.Color.Red;
                Connect.Text = "Connect";
                //  Clear the values in the "Values" section when the PLC is disconnected
                xPosVal.Text = "";
                yPosVal.Text = "";
                zPosVal.Text = "";
                feedRateVal.Text = "";
                cycleStartVal.Text = "";
                cycleStopVal.Text = "";
                startMovelVal.Text = "";
                userIPAddress.ReadOnly = false;
                Application.DoEvents();
            }
            else     //  Button operates as a connect button in this case
            {
                //   Verify that IP address entered in text box is valid
                bool validIP = ValidateIPv4(userIPAddress.Text);
                if (!validIP)
                {
                    MessageBox.Show("Invalid IP Address.");
                }
                else  //   Valid IP address
                {
                    //   Notify that connection is being attempted
                    ConnectionStatus.Text = "Connecting...";
                    ConnectionStatus.ForeColor = System.Drawing.Color.Orange;
                    Application.DoEvents();          //   Repaints ConnectionStatus.Text onto status strip in lower left corner

                    this.ipAddress = userIPAddress.Text;  // Since userIPAddress has a valid IP, store it into the ipAddress variable
                    
                    try    // Connecting to the MPiec
                    {   
                        MPiecComm.ConnectVariables(this.ipAddress);
                        connected = true;
                        ConnectionStatus.Text = "Connected";
                        ConnectionStatus.ForeColor = System.Drawing.Color.Green;
                        Connect.Text = "Disconnect";        // Change text on Connect button to say 'Disconnect'
                        userIPAddress.ReadOnly = true;
                    }
                    catch (System.NullReferenceException)
                    {
                        //   If the connection times out (~30 seconds) notify user that connection was unsuccessful
                        MessageBox.Show("Unable to connect to " + this.ipAddress.ToString() + ". Make sure controller is on and ready to connect.");
                        ConnectionStatus.Text = "Disconnected";
                        ConnectionStatus.ForeColor = System.Drawing.Color.Red;
                    }
                }
            }
        }

        //   This method is passed an IP Address as a string and returns a bool indicating whether it is a valid IPv4 address
        public static bool ValidateIPv4(string ipString)
        {
            if (ipString.Count(c => c == '.') != 3) return false;
            return IPAddress.TryParse(ipString, out IPAddress address);
        }

        //   Controls what happens when "Send to controller" is clicked
        private void send_Click(object sender, EventArgs e)
        {
            if (connected)
            {
                myStruct MyData = MPiecComm.ReadStruct();    // Reads current values from the structure in MotionWorks IEC
                

                //   Check if there is text in the feed box when button is clicked
                if (feedEdit.Text.Count() > 0)
                {
                    try
                    {
                        MyData.Feed = double.Parse(feedEdit.Text);
                        MPiecComm.WriteStruct(MyData);  
                    }
                    catch (System.FormatException ex)
                    {
                        Console.WriteLine("Not a valid double (LREAL).");
                        MessageBox.Show("Please enter a valid double (LREAL).");
                    }
                }

                //   Check if there is text in the cycle start box when button is clicked
                if (startEdit.Text.Count() > 0)
                {
                    try
                    {
                        //   Allow user to enter t or T for true, or f or F for false
                        if ((startEdit.Text == "t") || (startEdit.Text == "T") || (startEdit.Text == "1"))
                            startEdit.Text = "True";
                        if ((startEdit.Text == "f") || (startEdit.Text == "F") || (startEdit.Text == "0"))
                            startEdit.Text = "False";
                        MyData.CycleStart = bool.Parse(startEdit.Text);
                        MPiecComm.WriteStruct(MyData);  
                    }
                    catch (System.FormatException ex)
                    {
                        Console.WriteLine("Not a valid bool.");
                        MessageBox.Show("Please enter a valid bool.");
                    }
                }

                //   Check if there is text in the cycle stop box when button is clicked
                if (stopEdit.Text.Count() > 0)
                {
                    try
                    { 
                         //   Allow user to enter t or T for true, or f or F for false
                        if ((stopEdit.Text == "t") || (stopEdit.Text == "T") || (stopEdit.Text == "1"))
                            stopEdit.Text = "True";
                        if ((stopEdit.Text == "f") || (stopEdit.Text == "F") || (stopEdit.Text == "0"))
                            stopEdit.Text = "False";
                        MyData.CycleStop = bool.Parse(stopEdit.Text);
                        MPiecComm.WriteStruct(MyData);  
                    }
                    catch (System.FormatException ex)
                    {
                        Console.WriteLine("Not a valid bool.");
                        MessageBox.Show("Please enter a valid bool.");
                    }
                }

                if (startMoveEdit.Text.Count() > 0)
                {
                    try
                    {
                        //   Allow user to enter t or T for true, or f or F for false
                        if ((startMoveEdit.Text == "t") || (startMoveEdit.Text == "T") || (startMoveEdit.Text == "1"))
                            startMoveEdit.Text = "True";
                        if ((startMoveEdit.Text == "f") || (startMoveEdit.Text == "F") || (startMoveEdit.Text == "0"))
                            startMoveEdit.Text = "False";
                        MyData.StartMove = bool.Parse(startMoveEdit.Text);
                        MPiecComm.WriteStruct(MyData);  
                    }
                    catch (System.FormatException ex)
                    {
                        Console.WriteLine("Not a valid bool.");
                        MessageBox.Show("Please enter a valid bool.");
                    }
                }

                //  Clear out text boxes after pressing Update
                feedEdit.Text = "";
                startEdit.Text = "";
                stopEdit.Text = "";
                startMoveEdit.Text = "";
            }
        }

        //   This method occurs if the user hits enter while in the IP address text box 
        //   It simulates the Connect/Disconnect button in the UI being pressed 
        private void CheckEnterKeyPressConnect(object sender, System.Windows.Forms.KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Return)

            {
                Connect_Click(sender, e);
            }
        }

        //   This method occurs if the user hits enter while in the Start Move text box 
        //   It simulates the Send to Controller button in the UI being pressed 
        private void GenericDataKeyPress(object sender, System.Windows.Forms.KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Return)

            {
                send_Click(sender, e);
            }
        }

        /* the following methods are used to update the text in the "Online Value" text boxes
         * they are called from the OnTimer() method */

        delegate void SetTextCallback(string text);     // SetTextCallback enables asynchronous calls for setting the Text property of a TextBox

        //   Updates text in x position text box if new data is detected
        private void SetTextX(string text)
        {
            //   InvokeRequired compares the thread ID of the calling thread to the thread ID of the creating thread.
            //   If these threads are different, it returns true.
            if (this.xPosVal.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetTextX);
                this.Invoke(d, new object[] { text });
            }
            else    //  'text' differs from 'this.xPosVal.Text', so the text box is updated
            {
                this.xPosVal.Text = text;
            }
        }

        //  Updates text in y position text box if new data is detected
        private void SetTextY(string text)
        {
            //   InvokeRequired compares the thread ID of the calling thread to the thread ID of the creating thread.
            //   If these threads are different, it returns true.
            if (this.yPosVal.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetTextY);
                this.Invoke(d, new object[] { text });
            }
            else    // 'text' differs from 'this.yPosVal.Text', so the text box is updated
            {
                this.yPosVal.Text = text;
            }
        }

        //   Updates text in z position text box if new data is detected
        private void SetTextZ(string text)
        {
            //   InvokeRequired compares the thread ID of the calling thread to the thread ID of the creating thread.
            //   If these threads are different, it returns true.
            if (this.zPosVal.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetTextZ);
                this.Invoke(d, new object[] { text });
            }
            else    // 'text' differs from 'this.zPosVal.Text', so the text box is updated
            {
                this.zPosVal.Text = text;
            }
        }

        //   Updates text in feed rate text box if new data is detected
        private void SetTextFeed(string text)
        {
            //   InvokeRequired compares the thread ID of the calling thread to the thread ID of the creating thread.
            //   If these threads are different, it returns true.
            if (this.feedRateVal.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetTextFeed);
                this.Invoke(d, new object[] { text });
            }
            else    // 'text' differs from 'this.feedRateVal.Text', so the text box is updated
            {
                this.feedRateVal.Text = text;
            }
        }

        //   Updates text in cycle start text box if new data is detected
        private void SetTextStart(string text)
        {
            //   InvokeRequired compares the thread ID of the calling thread to the thread ID of the creating thread.
            //   If these threads are different, it returns true.
            if (this.cycleStartVal.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetTextStart);
                this.Invoke(d, new object[] { text });
            }
            else    // 'text' differs from 'this.cycleStartVal.Text', so the text box is updated
            {
                this.cycleStartVal.Text = text;
            }
        }

        //   Updates text in cycle stop text box if new data is detected
        private void SetTextStop(string text)
        {
            //   InvokeRequired compares the thread ID of the calling thread to the thread ID of the creating thread.
            //   If these threads are different, it returns true.
            if (this.cycleStopVal.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetTextStop);
                this.Invoke(d, new object[] { text });
            }
            else    // 'text' differs from 'this.cycleStopVal.Text', so the text box is updated
            {
                this.cycleStopVal.Text = text;
            }
        }

        //   Updates text in Start Move text box if new data is detected
        private void SetTextStartMove(string text)
        {
            //   InvokeRequired compares the thread ID of the calling thread to the thread ID of the creating thread.
            //   If these threads are different, it returns true.
            if (this.startMovelVal.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetTextStartMove);
                this.Invoke(d, new object[] { text });
            }
            else    // 'text' differs from 'this.startMoveVal.Text', so the text box is updated
            {
                this.startMovelVal.Text = text;
            }
        }

    }
}
